﻿using System;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Web.Http.Controllers;
using System.Web.Http.Filters;
using Kreta.Core.Configuratiaton;

namespace Kreta.Naplo.WebApi.FilterAttributes
{
    internal class ApiKeyAuthorizationAttribute : AuthorizationFilterAttribute, IOverrideFilter
    {
        public Type FiltersToOverride => typeof(IAuthorizationFilter);

        public override void OnAuthorization(HttpActionContext actionContext)
        {
            if (!AccessGrantedByApiKey(actionContext.Request.Headers))
            {
                actionContext.Response = new HttpResponseMessage(HttpStatusCode.Forbidden)
                {
                    Content = new StringContent("permission_denied")
                };
            }
        }

        private static bool AccessGrantedByApiKey(HttpRequestHeaders headers)
        {
            if (headers is null)
            {
                throw new ArgumentNullException(nameof(headers));
            }

            if (ApiKeyConfiguration.Instance.Enabled)
            {
                if (headers.TryGetValues(nameof(ApiKeyConfiguration.ApiKey), out var ApiKeys))
                {
                    string targetApiKey = ApiKeys.FirstOrDefault();

                    if (!string.IsNullOrWhiteSpace(targetApiKey) && targetApiKey == ApiKeyConfiguration.Instance.ApiKey)
                    {
                        return true;
                    }
                }
            }
            else
            {
                return true;
            }

            return false;
        }
    }
}
