﻿using System;
using Kreta.BusinessLogic.Interfaces;
using Kreta.Client.CoreApi;
using Kreta.Naplo.Dto.V3.Interfaces;
using Kreta.Naplo.WebApi.V3.Documentation.Helper;
using Kreta.Naplo.WebApi.V3.Documentation.MockServices;
using Swashbuckle.Examples;

namespace Kreta.Naplo.WebApi.V3.Documentation
{

    internal class DummyListExampleProvider<TController, TDto> : IExamplesProvider where TController : new() where TDto : IDtoListDocumentation, new()
    {
        public object GetExamples()
        {
            object response = ExampleHelper.TryGetExampleFeatureDefaultExampleModel();
            if (response != null)
            {
                return response;
            }

            TDto dto = new TDto();
            object[] parameterArray = dto.ListExample.RequestParameter == null
                ? new object[] { }
                : new[] { dto.ListExample.RequestParameter };

            ExampleHelper.MockUser(dto.ListExample.MockUserName);

            return new
            {
                RequestExample = dto.ListExample.RequestParameter,
                RequestExampleUser = dto.ListExample.MockUserName,
                ResponseExample = typeof(TController).GetMethod(dto.ListExample.MethodName).Invoke(new TController(), parameterArray)
            };
        }
    }

    internal class DummyListExampleProviderWithDependencyInjection<TController, TDto> : IExamplesProvider where TDto : IDtoListDocumentation, new()
    {
        private readonly IFileServiceHelper _fileServiceHelper = new FileServiceHelperMock();

        private readonly ICoreApiClient _coreApiClient = new CoreApiClientMock();

        public object GetExamples()
        {
            object response = ExampleHelper.TryGetExampleFeatureDefaultExampleModel();
            if (response != null)
            {
                return response;
            }

            TDto dto = new TDto();
            object[] parameterArray = dto.ListExample.RequestParameter == null
                ? new object[] { }
                : new[] { dto.ListExample.RequestParameter };

            ExampleHelper.MockUser(dto.ListExample.MockUserName);

            return new
            {
                RequestExample = dto.ListExample.RequestParameter,
                RequestExampleUser = dto.ListExample.MockUserName,
                ResponseExample = typeof(TController).GetMethod(dto.ListExample.MethodName).Invoke((TController)Activator.CreateInstance(typeof(TController), _fileServiceHelper, _coreApiClient), parameterArray)
            };
        }
    }
}
